@@ -9,7 +9,7 @@ use HTML::HTMLDoc::PDF;
 use vars qw(@ISA $VERSION);
 
 @ISA = qw();
-$VERSION = '0.06';
+$VERSION = '0.07';
 my $DEBUG = 0;
 
 ###############
@@ -57,10 +57,35 @@ sub _init {
 	$self->set_charset('iso-8859-1');
 	$self->_set_doc_config('quiet');
 	$self->set_output_format('pdf');
+	# standard-header and footer
+	$self->set_footer('.', '1', '.');
+	$self->set_header('.', 't', '.');
+
+}
+
+###############
+# Store or get a global configuration value
+# testet
+# param: key:STRING, value:STRING
+# return: 1
+###############
+sub _config {
+	my $self = shift;
+	my $key = shift;
+	my $value = shift;
+
+	my $ret;
+	if (defined $value) {
+		$self->{'config'}->{$key} = $value;
+	} else {
+		$ret = $self->{'config'}->{$key};
+	}
+	return $ret;
 }
 
 ###############
 # stores a specific value for formating the outputdoc
+# testet
 # param: key:STRING, value:STRING
 # return: 1
 ###############
@@ -69,26 +94,55 @@ sub _set_doc_config {
 	my $key = shift;
 	my $value = shift;
 
-	$self->{'doc_config'}->{$key} = $value;
+	if (ref($value) && (ref($value) eq 'ARRAY') ) {
+		# the value is an array, store it in an array too
+		if ( !$self->{'doc_config'}->{$key} || ref($self->{'doc_config'}->{$key}) ne 'ARRAY') {
+			# create a new array
+			$self->{'doc_config'}->{$key} = [];
+		}
+		foreach my $single_value(@{$value}) {
+			push(@{$self->{'doc_config'}->{$key}}, $single_value);
+		}
+	} else {
+		$self->{'doc_config'}->{$key} = $value;
+	}
+
 	return 1;
 }
 
 ###############
 # deletes a specific config
+# testet
 # param: key:STRING
 # return: value:STRING
 ###############
 sub _delete_doc_config {
 	my $self = shift;
 	my $key = shift;
+	my $value = shift;
+
 	if (exists $self->{'doc_config'}->{$key}) {
-		delete $self->{'doc_config'}->{$key};
+		my $set_value = $self->_get_doc_config($key);
+		if ( (ref($set_value) eq 'ARRAY') && $value ) {
+			# remove specific value only
+			# find the position of the value
+			for(my $i=0; $i<@{$set_value}; $i++) {
+				if ( $set_value->[$i] eq $value ) {
+					splice(@{$set_value}, $i, 1);
+					last;
+				}
+			}
+		} else {
+			# delete the singlevalue
+			delete $self->{'doc_config'}->{$key};
+		}
 	}
 }
 
 
 ###############
 # tells a specific value for formating the outputdoc
+# testet
 # param: key:STRING
 # return: value:STRING
 ###############
@@ -114,6 +168,7 @@ sub _get_doc_config_keys {
 ###############
 # tests if the parameter exists in the array
 # of allowed params
+# testet
 # param: key:STRING, \@allowed
 # return: 1/0
 ###############
@@ -124,13 +179,13 @@ sub _test_params {
 
 	my $ok = 0;
 	foreach my $aparam (@{$allowed}) {
-		if ($param eq $aparam) {
+		if (lc($param) eq lc($aparam)) {
 			$ok=1;
 			last;
 		}
 	}
 
-	return 1;
+	return $ok;
 }
 
 
@@ -141,6 +196,7 @@ sub _test_params {
 
 ###############
 # sets the size of the pages - default: a4
+# testet
 # param: letter, a4, WxH{in,cm,mm}
 # return: 1/0
 ###############
@@ -157,6 +213,11 @@ sub set_page_size {
     return 1;
 }
 
+###############
+# reads out the page-size
+# param: letter, a4, WxH{in,cm,mm}
+# return: 1/0
+###############
 sub get_page_size {
 	my $self = shift;
 	return $self->_get_doc_config('size');
@@ -164,6 +225,7 @@ sub get_page_size {
 
 ###############
 # sets the master-password of the doc
+# testet
 # param: password:STRING
 # return: 1/0
 ###############
@@ -176,6 +238,7 @@ sub set_owner_password {
 
 ###############
 # sets the user-password of the doc
+# testet
 # param: password:STRING
 # return: 1/0
 ###############
@@ -189,27 +252,59 @@ sub set_user_password {
 # all,annotate,copy,modify,print,no-annotate,no-copy,no-modify,no-print,none
 ###############
 # sets the master-password of the doc
+# testet
 # param: password:STRING
 # return: 1/0
 ###############
 sub set_permissions {
 	my $self = shift;
-	my $value = shift;
+	my @values = @_;
+
+	my $thiskey = 'permissions';
 
 	my @allowed = ('all','annotate','copy','modify','print','no-annotate','no-copy','no-modify','no-print','none');
 	# test the set value
-	if ($self->_test_params(lc($value), \@allowed)) {
-		# wrong permission set
-		$self->error("wrong permission set: $value");
+	if ($#values==-1) {
+		$self->error("wrong permission set: no values");
 		return 0;
 	}
+	for( my $i=0; $i<=$#values; $i++ ) {
+		$values[$i] = lc($values[$i]);
+		if ( !$self->_test_params($values[$i], \@allowed) ) {
+			# wrong permission set
+			$self->error("wrong permission set: $values[$i]");
+			return 0;
+		}
+	}
 
-	$self->_set_doc_config('permissions', $value);
+	foreach my $value(@values) {
+		# take care of the combination of the options
+		if ( $value eq 'all' ) {
+			# delete all
+			$self->_delete_doc_config($thiskey);
+		} elsif( $value eq 'none' ) {
+			# delete all
+			$self->_delete_doc_config($thiskey);
+		} else {
+			# delete the corresponding flag
+			if ( $value =~/^no-(.+)/ ) {
+				my $key = $1;
+				$self->_delete_doc_config($thiskey, $key);
+			} else {
+				$self->_delete_doc_config($thiskey, "no-$value");
+			}
+		}
+
+		$self->_set_doc_config('permissions', [$value]);
+	}
+	# enable encryption since without it there is no effect.
+	$self->enable_encryption();
 	return 1;
 }
 
 ###############
 # sets the pages to landscape
+# testet
 # param: -
 # return: 1/0
 ###############
@@ -217,12 +312,13 @@ sub landscape {
 	my $self = shift;
 
 	$self->_set_doc_config('landscape', '');
-	$self->_delete_doc_config('portait');
+	$self->_delete_doc_config('portrait');
 	return 1;
 }
 
 ###############
 # sets the pages to landscape
+# testet
 # param: -
 # return: 1/0
 ###############
@@ -260,8 +356,110 @@ sub no_title {
 	return 1;
 }
 
+###############
+# sets the footer
+# testet
+# param: left:CHAR, center:CHAR, right:CHAR
+# return: 1/0
+###############
+sub set_footer {
+	my $self = shift;
+	my $left = shift;
+	my $center = shift;
+	my $right = shift;
+
+	my @allowed = ('.', ':', '/', '1', 'a', 'A', 'c', 'C', 'd', 'D', 'h', 'i', 'I', 'l', 't', 'T');
+	if (!$self->_test_params($left, \@allowed) ) {
+		$self->error("wrong left-footer-option: $left");
+		return 0;
+	}
+	if (!$self->_test_params($center, \@allowed) ) {
+		$self->error("wrong center-footer-option: $left");
+		return 0;
+	}
+	if (!$self->_test_params($right, \@allowed) ) {
+		$self->error("wrong right-footer-option: $left");
+		return 0;
+	}
+
+	$self->_set_doc_config('footer', "${left}${center}${right}");
+
+	return 1;
+}
+
+###############
+# sets the footer
+# testet
+# param: left:CHAR, center:CHAR, right:CHAR
+# return: 1/0
+###############
+sub set_header {
+	my $self = shift;
+	my $left = shift;
+	my $center = shift;
+	my $right = shift;
+
+	my @allowed = ('.', ':', '/', '1', 'a', 'A', 'c', 'C', 'd', 'D', 'h', 'i', 'I', 'l', 't', 'T');
+	if (!$self->_test_params($left, \@allowed) ) {
+		$self->error("wrong left-header-option: $left");
+		return 0;
+	}
+	if (!$self->_test_params($center, \@allowed) ) {
+		$self->error("wrong center-header-option: $left");
+		return 0;
+	}
+	if (!$self->_test_params($right, \@allowed) ) {
+		$self->error("wrong right-header-option: $left");
+		return 0;
+	}
+
+	$self->_set_doc_config('header', "${left}${center}${right}");
+
+	return 1;
+}
+
+###############
+# turns the links on
+# param: -
+# return: 1/0
+###############
+sub links {
+	my $self = shift;
+
+	$self->_set_doc_config('links', '');
+	$self->_delete_doc_config('no-links');
+	return 1;
+}
+
+###############
+# turns the links off
+# param: -
+# return: 1/0
+###############
+sub no_links {
+	my $self = shift;
+
+	$self->_set_doc_config('no-links', '');
+	$self->_delete_doc_config('links');
+	return 1;
+}
+
+###############
+# sets the search path for files in a document
+# param: -
+# return: 1/0
+###############
+sub path {
+	my $self = shift;
+    my $sp = shift;
+
+	$self->_set_doc_config('path', $sp);
+	return 1;
+}
+
 ###############
 # sets the right margin
+# testet
 # param: margin|NUM, messure:in,cm,mm
 # return: 1/0
 ###############
@@ -274,6 +472,7 @@ sub set_right_margin {
 
 ###############
 # sets the right margin
+# testet
 # param: margin|NUM, messure:in,cm,mm
 # return: 1/0
 ###############
@@ -327,17 +526,63 @@ sub _set_margin {
 
 ###############
 # sets the color of the body
+# testet
 # param: color:hex
 # return: 1/0
 ###############
 sub set_bodycolor {
 	my $self = shift;
-	my $color = shift;
-	return $self->_set_doc_config('bodycolor', $color);
+
+	my $ret;
+	my $color = $self->_test_color(@_);
+	if (!$color) {
+		$self->error("wrong value set for bodycolor");
+		$ret = 0;
+	} else {
+		$ret = $self->_set_doc_config('bodycolor', $color);
+	}
+
+	return $ret;
+}
+
+###############
+# internal method for testing and converting colors
+# testet
+# param: color:hex || color: rgb || color: name
+# return: color:hex
+###############
+sub _test_color {
+	my $self = shift;
+	my @colors = @_;
+	my $ret;
+
+	if( (@colors == 1) && $colors[0]=~/^#[0-9a-f]{6}$/i ) {
+		# got hex-color
+		return $colors[0];
+	} elsif( @colors==3 ) {
+		# 3 input values, test if regular rgb is given
+		my ($r, $g, $b) = @colors;
+		if ($r=~/^\d{1,3}$/ && $g=~/^\d{1,3}$/ && $b=~/^\d{1,3}$/
+			&& $r>=0 && $r <=255 && $b>=0 && $b<=255 && $g>=0 && $g<=255) {
+				$ret = sprintf("#%02x%02x%02x", $r, $g, $b);
+		}
+	} elsif( @colors==1 ) {
+		foreach my $c( qw(red green blue cyan magenta yellow darkRed
+			darkGreen darkBlue darkCyan darkMagenta darkYellow white
+			lightGray gray darkGray black) ) {
+				if ($c eq $colors[0]) {
+					$ret = $c;
+					last;
+				}
+			}
+	}
+
+	return $ret;
 }
 
 ###############
 # sets the default font for the body
+# testet
 # param: fontface:STRING
 # return: 1/0
 ###############
@@ -345,14 +590,35 @@ sub set_bodyfont {
 	my $self = shift;
 	my $font = shift;
 
+	my $ret = 0;
 	my @allowed = qw(Arial Courier Helvetica Monospace Sans-Serif Serif Symbol Times);
 	if ( !$self->_test_params($font, \@allowed) ) {
 		$self->error("illegal font set $font");
+	} else {
+		$self->_set_doc_config('bodyfont', $font);
+		$ret = 1;
 	}
 
-	return $self->_set_doc_config('bodyfont', $font);
+	return $ret;
 }
 
+###############
+# sets the font size for body text
+# param: size:NUM
+# return: 1/0
+###############
+sub set_fontsize {
+	my $self = shift;
+	my $fsize = shift;
+	if ($fsize =~ /^\d+(\.\d+){0,1}$/) {
+		return $self->_set_doc_config('fontsize', $fsize);
+	} else {
+		$self->error("illegal font size $fsize");
+		return 0;
+	}
+}
+
+
 ###############
 # takes an image-filename that is used as background
 # for all Pages
@@ -433,6 +699,30 @@ sub set_charset {
 	return 1;
 }
 
+###############
+# embedding the used fonts into the pdf-file
+# testet
+# param:
+# return: 1/0
+###############
+sub embed_fonts {
+	my $self = shift;
+	$self->_set_doc_config('embedfonts', '');
+	return 1;
+}
+
+###############
+# no font embedding
+# testet
+# param:
+# return: 1/0
+###############
+sub no_embed_fonts {
+	my $self = shift;
+	$self->_delete_doc_config('embedfonts');
+	return 1;
+}
+
 ###############
 # turns colors on in doc
 # param: charset
@@ -448,7 +738,7 @@ sub color_on {
 
 ###############
 # turns colors on in doc
-# param: 
+# param:
 # return: 1/0
 ###############
 sub color_off {
@@ -533,9 +823,42 @@ sub set_html_content {
 ###############
 sub get_html_content {
 	my $self = shift;
+
+	if (ref($self->{'html'}) eq 'SCALAR') {
+		return ${$self->{'html'}};
+	}
+
 	return $self->{'html'};
+}           
+
+###############
+# sets the filename of the html-page that should be rendered
+# param: input_file:STRING
+# return: 1/0
+###############
+sub set_input_file {
+	my $self = shift;
+	my $infile = shift;
+
+    if (-f $infile) {
+        $self->{'input_file'} = $infile;
+        $self->{'config'}->{'mode'} = 'file';
+        return 1;
+    }
+    return 0;
 }
 
+###############
+# returns the input htmlfile
+# param: -
+# return: input_file:STRING
+###############
+sub get_input_file {
+	my $self = shift;
+	return $self->{'input_file'};
+}
+
+
 ###############
 # private: opens a temporary file and sets the
 # html-content in
@@ -544,9 +867,11 @@ sub get_html_content {
 ###############
 sub _prepare_input_file {
 	my $self = shift;
-
+                 
 	my $i=0;
-	my $filename;   
+	my $filename;
+	return $filename if (defined ($filename = $self->{'input_file'}));
+
 	while($i<1000) {
 		my $randpart = int(rand(1000));
 		$filename = $self->{'config'}->{'tmpdir'} . "/htmldoc$randpart.html";
@@ -578,7 +903,8 @@ sub _prepare_input_file {
 ###############
 sub _cleanup {
 	my $self = shift;
-	unlink($self->{'config'}->{'tmpfile'});
+	unlink($self->{'config'}->{'tmpfile'})
+		if ( (defined $self->{'config'}->{'tmpfile'}) && (-f $self->{'config'}->{'tmpfile'}) );
 }
 
 ###############
@@ -592,7 +918,8 @@ sub generate_pdf {
 	my $params = $self->_build_parameters();
 	my $pdf;
 
-	if ($self->{'config'}->{'mode'} eq 'ipc') {
+	#if ($self->{'config'}->{'mode'} eq 'ipc') {
+	if ($self->_config('mode') eq 'ipc') {
 		# we are in normale Mode, use IPC
 		my ($pid, $error);
     	($pid,$pdf,$error) = $self->_run("htmldoc  $params --webpage -", $self->get_html_content());
@@ -611,6 +938,7 @@ sub generate_pdf {
 
 ###############
 # generates a string for the configuration of htmldoc
+# testet
 # param: -
 # return: params:STRING
 ###############
@@ -621,7 +949,15 @@ sub _build_parameters {
 
 	foreach my $key($self->_get_doc_config_keys()) {
 		my $value = $self->_get_doc_config($key) || '';
-		$paramstring .= " --$key $value";
+		if ( ref($value) eq 'ARRAY' ) {
+			# an array, set the option multiple
+			foreach my $single_v(@{$value}) {
+				$paramstring .= " --$key $single_v";
+			}
+		} else {
+			$paramstring .= " --$key $value";
+		}
+
 	}
 	return $paramstring;
 }
@@ -665,7 +1001,7 @@ sub _run {
 
 
 ###############
-# set or retrieve an accured error
+# set or retrieve an occurred error
 # param: -
 # return: pdf:STRING
 ###############
@@ -689,7 +1025,7 @@ __END__
 
 =head1 NAME
 
-HTML::HTMLDoc - Perl interface to the htmldoc programm for producing PDF-Files from HTML-Content
+HTML::HTMLDoc - Perl interface to the htmldoc program for producing PDF-Files from HTML-Content
 
 =head1 SYNOPSIS
 
@@ -698,6 +1034,7 @@ HTML::HTMLDoc - Perl interface to the htmldoc programm for producing PDF-Files f
   my $htmldoc = new HTML::HTMLDoc();
 
   $htmldoc->set_html_content(qq~<html><body>A PDF file</body></html>~);
+  # $htmldoc->set_input_file($filename); # alternative to use a present file from your fs
 
   my $pdf = $htmldoc->generate_pdf();
 
@@ -715,8 +1052,8 @@ parameters of HTMLDoc are supported.
 
 You need to have HTMLDoc installed before installing this module.
 
-All the pdf-Methods return true for success or false for failure. You can test if errors 
-accured by calling the error-method.
+All the pdf-Methods return true for success or false for failure. You can test if errors
+occurred by calling the error-method.
 
 Normaly this module uses IPC::Open3 for communacation with the HTMLDOC process. However,
 in mod_perl-environments there appear problems with this module because the standard-output can not
@@ -757,7 +1094,7 @@ letter
 =item *
 WxH{in,cm,mm} eg '10x10cm'
 
-=back       
+=back
 
 
 =head2 set_owner_password($password)
@@ -808,8 +1145,34 @@ no-print
 =item *
 none
 
-=back
 
+setting one of this flags automatically enables the document-encryption ($htmldoc->enable_encryption())
+for you, because setting permissions will have no effect without it.
+
+Setting 'all' and 'none' will delete all other previously set options. You can set multiple options if
+you need, eg.:
+
+$htmldoc->set_permissions('no-copy');
+$htmldoc->set_permissions('no-modify');
+
+this one will do the same:
+$htmldoc->set_permissions('no-copy', 'no-modify');
+
+=back 
+
+=head2 links()
+
+turns link processing on.
+
+
+=head2 no_links()
+
+turns the links off.
+ 
+
+=head2 path()
+
+specify the search path for files in a document
 
 
 =head2 landscape()
@@ -817,7 +1180,7 @@ none
 sets the format of the resulting pages to landscape
 
 
-=head2 portait()
+=head2 portrait()
 
 sets the format of the resulting pages to portrait
 
@@ -854,27 +1217,33 @@ set the top margin. $margin is a INT, $messure one of 'in', 'cm' or 'mm'.
 
 =head2 set_bodycolor($color)
 
-Sets the background of all pages to this background color. $color is a hex-coded color-value (eg. #FFFFFF).
+Sets the background of all pages to this background color. $color is a hex-coded color-value (eg. #FFFFFF),
+a rgb-value (eg set_bodycolor(0,0,0) for black) or a color name (eg. black)
 
 
 =head2 set_bodyfont($font)
 
-Sets the default font of the content. Currently the following fonts are supported:  
+Sets the default font of the content. Currently the following fonts are supported:
 
 Arial Courier Helvetica Monospace Sans-Serif Serif Symbol Times
+ 
+
+=head2 set_fontsize($fsize)
+
+Sets the default font size for the body text.
 
 
 =head2 set_bodyimage($image)
 
 Sets the background image for the document. $image is the path to the image in your filesystem.
-                      
+
 
 =head2 set_browserwidth($width)
 
 specifies the browser width in pixels. The browser width is used to scale images and pixel measurements when generating PostScript and PDF files. It does not affect the font size of text.
 
 The default browser width is 680 pixels which corresponds roughly to a 96 DPI display. Please note that your images and table sizes are equal to or smaller than the browser width, or your output will overlap or truncate in places.
-         
+
 
 =head2 set_compression($level)
 
@@ -889,7 +1258,7 @@ specifies the initial viewing mode of the document. $mode is one of:
 
 =over 4
 
-=item * 
+=item *
 document - the document pages are displayed in a normal window
 
 =item *
@@ -900,14 +1269,14 @@ fullscreen - the document pages are displayed on the entire screen
 
 =back
 
-   
+
 =head2 set_charset($charset)
 
 defines the charset for the output document. The following charsets are currenty supported:
 cp-874 cp-1250 cp-1251 cp-1252 cp-1253 cp-1254 cp-1255 cp-1256 cp-1257 cp-1258
-iso-8859-1 iso-8859-2 iso-8859-3  iso-8859-4 iso-8859-5 iso-8859-6 iso-8859-7 
+iso-8859-1 iso-8859-2 iso-8859-3  iso-8859-4 iso-8859-5 iso-8859-6 iso-8859-7
 iso-8859-8 iso-8859-9 iso-8859-14 iso-8859-15 koi8-r
-                                    
+
 
 =head2 color_on()
 
@@ -970,7 +1339,8 @@ ps3
 
 =head2 set_html_content($html)
 
-this is the function to set the html-content as a scalar.
+this is the function to set the html-content as a scalar. See set_input_file($filename)
+to use a present file from your filesystem for input
 
 
 =head2 get_html_content()
@@ -978,6 +1348,117 @@ this is the function to set the html-content as a scalar.
 gives back the previous set html-content.
 
 
+=head2 set_input_file($input_filename)
+
+this is the function to set the input file name.  It will also switch the
+operational mode to 'file'.
+
+
+=head2 get_input_file()
+
+gives back the previous set input file name.
+
+
+=head2 set_header($left, $center, $right)
+
+defines the data that should be displayed in header. One can choose from the following chars for each left,
+center and right:
+
+=over 4
+
+=item *
+B<.> A period indicates that the field should be blank.
+
+
+=item *
+B<:> A colon indicates that the field should contain the current and total number of pages in the chapter (n/N).
+
+
+=item *
+B</> A slash indicates that the field should contain the current and total number of pages (n/N).
+
+
+=item *
+B<1> The number 1 indicates that the field should contain the current page number in decimal format (1, 2, 3, ...)
+
+
+=item *
+B<a> A lowercase "a" indicates that the field should contain the current page number using lowercase letters.
+
+
+=item *
+B<A> An uppercase "A" indicates that the field should contain the current page number using UPPERCASE letters.
+
+
+=item *
+B<c> A lowercase "c" indicates that the field should contain the current chapter title.
+
+
+=item *
+B<C> An uppercase "C" indicates that the field should contain the current chapter page number.
+
+
+=item *
+B<d> A lowercase "d" indicates that the field should contain the current date.
+
+
+=item *
+B<D> An uppercase "D" indicates that the field should contain the current date and time.
+
+
+=item *
+B<h> An "h" indicates that the field should contain the current heading.
+
+
+=item *
+B<i> A lowercase "i" indicates that the field should contain the current page number in lowercase roman numerals (i, ii, iii, ...)
+
+
+=item *
+B<I> An uppercase "I" indicates that the field should contain the current page number in uppercase roman numerals (I, II, III, ...)
+
+
+=item *
+B<l> A lowercase "l" indicates that the field should contain the logo image.
+
+
+=item *
+B<t> A lowercase "t" indicates that the field should contain the document title.
+
+
+=item *
+B<T> An uppercase "T" indicates that the field should contain the current time.
+
+
+=back
+
+Example:
+
+Setting the header to contain the title left, nothing in center and actual pagenumber right do the follwing
+
+$htmldoc-E<gt>set_header('t', '.', '1');
+
+
+
+
+=head2 set_footer($left, $center, $right)
+
+defines the data that should be displayed in footer. See set_header() for details setting the left, center and right
+value.
+
+
+
+=head2 embed_fonts()
+
+specifies that fonts should be embedded in PostScript and PDF output. This is especially useful when generating documents in character sets other than ISO-8859-1.
+
+
+
+=head2 no_embed_fonts()
+
+turn the font-embedding previously enabled by embed_fonts() off.
+
+
 =head2 generate_pdf()
 
 generates the output-document. Returns a instance of HTML::HTMLDoc::PDF. See the perldoc of this class
@@ -986,7 +1467,7 @@ for details
 
 =head2 error()
 
-in scalar content returns the last error that occured, in list context returns all errors that accured.
+in scalar content returns the last error that occurred, in list context returns all errors that occurred.
 
 
 =head2 EXPORT
@@ -1003,6 +1484,20 @@ Michael Frankl - mfrankl@seibert-media.de
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself
+               
+
+=head1 CREDITS
+
+Thanks very much to:
+
+Rajat Bhatia
+
+Keith W. Sheffield
+
+Christoffer Landtman
+
+
+for suggestions and bug fixes.
 
 
 =head1 SEE ALSO
